<template><div>加载中...</div></template>

<script lang="ts">
// import '/static/sdk/sdk.js';
// import '/static/sdk/sdk.css';
// import '/static/sdk/iconfont.css';
import { accessSessionKey, handleNoAuth } from '/@/utils/request';
import { NO_AUTH_API_LIST } from '/@/api/ai/chat';
import { LOGIN_URL, TEL_LOGIN_URL } from '/@/api/ai/user';
import { MAIN_URL } from '/@/constants';
import router from '/@/router/index';

import { Local } from '/@/utils/storage';

// 可以不引用, 如果你不想要任何辅助类样式的话 (比如 `m-t-xs` 这种)
// https://aisuda.bce.baidu.com/amis/zh-CN/style/index
// import 'amis/sdk/helper.css';
import qs from 'qs';

function loadScript(src, callback) {
	const script = document.createElement('script');
	script.setAttribute('type', 'text/javascript');
	script.setAttribute('src', src);
	script.onload = () => callback();
	script.onerror = () => callback(new Error(`Failed to load ${src}`));
	document.body.appendChild(script);
}

function loadStyles(styles) {
	for (const path of styles) {
		const style = document.createElement('link');
		style.setAttribute('rel', 'stylesheet');
		style.setAttribute('type', 'text/css');
		style.setAttribute('href', path);
		document.head.appendChild(style);
	}
}

function loadSDK() {
	return new Promise((resolve, reject) => {
		if (window.amisRequire) {
			resolve();
			return;
		}
		loadStyles(['/static/amis/sdk/sdk.css', '/static/amis/sdk/helper.css', '/static/amis/sdk/iconfont.css']);
		loadScript('/static/amis/sdk/sdk.js', (err) => {
			if (err) {
				reject(err);
				return;
			}
			resolve();
		});
	});
}

export default {
	name: 'AMISRenderer',
	components: {},
	props: {
		schema: {
			type: Object,
			default: () => ({
				type: 'page',
				body: 'Hello World!',
			}),
		},
		locals: {
			type: Object,
			default: () => ({}),
		},
		props: {
			type: Object,
			default: () => ({}),
		},
		env: {
			type: Object,
			default: () => ({}),
		},
	},
	data() {
		return {
			// 这里面的数据所有 amis 页面都可以获取到
			// 可以用来放一下公共数据，比如用户信息等
			// 不要放受控数据，受控数据应该通过 data 下发
			context: {
				siteName: 'AMIS DEMO',
			},
			get location() {
				const current = router.currentRoute.value;
				return {
					pathname: current?.path,
					hash: current?.hash,
					query: current?.query,
					search: `?${qs.stringify(current.query)}`,
				};
				// return 'localtion'
			},
			loading: false,
			amisInstance: null,
			unmounted: false,
		};
	},

	watch: {
		locals: function () {
			this.updateProps();
		},
		props: function () {
			this.updateProps();
		},
		$route: function () {
			this.updateProps();
		},
	},
	async mounted() {
		try {
			this.loading = true;
			await loadSDK();
		} finally {
			this.loading = false;
		}
		if (this.unmounted) {
			return;
		}

		const scoped = amisRequire('amis/embed');
		const { normalizeLink } = amisRequire('amis');
		const instance = scoped.embed(
			this.$el,
			this.schema,
			{
				data: {
					...this.locals,
				},
				context: this.context,
				location: this.location,

				// todo 下发 location 对象
				...this.props,
			},
			{
				requestAdaptor(api) {
					// 支持异步，可以通过 api.mockResponse 来设置返回结果，跳过真正的请求发送
					// 此功能自定义 fetcher 的话会失效
					// api.context 中包含发送请求前的上下文信息
					// 获取本地的 token
					const accessSession = Local.get(accessSessionKey);
					if (!NO_AUTH_API_LIST.includes(api.url)) {
						if (accessSession) {
							// 将 token 添加到请求报文头中
							api.headers['hswatersession'] = accessSession;
						} else {
							if (api.url !== LOGIN_URL && api.url !== TEL_LOGIN_URL) {
								handleNoAuth(api.url);
								throw '权限验证失败';
							}
						}
					}
				

					api.url = `${MAIN_URL}${api.url}`;
					return api;
				},
				// 覆盖 amis env
				// 参考 https://aisuda.bce.baidu.com/amis/zh-CN/docs/start/getting-started#sdk
				jumpTo: (to, action) => {
					return;
					console.log("🚀 ~ to:", to)
					console.log("🚀 ~ action:", action)
					if (to === 'goBack') {
						return router.go(-1);
						
					}

					to = normalizeLink(to, this.location);

					if (action?.actionType === 'url') {
						action.blank === false ? router.push(to) : window.open(to);
						return;
					}

					// 主要是支持 nav 中的跳转
					if (action && to && action.target) {
						window.open(to, action.target);
						return;
					}

					if (/^https?:\/\//.test(to)) {
						window.location.replace(to);
					} else {
						router.push(to);
					}
				},

				updateLocation: (location, replace) => {
					// 禁止跳转
					return;
					console.log("🚀 ~ location:", location)
					console.log("🚀 ~ replace:", replace)
					if (location === 'goBack') {
						return router.go(-1);
					}

					location = normalizeLink(location, this.location);
					replace ? router.replace(location) : router.replace(location);
				},

				...this.env,
			},
			() => {
				this.$emit('ready', {
					instance,
				});
			}
		);

		this.amisInstance = instance;
	},

	methods: {
		updateProps() {
			this.amisInstance?.updateProps({
				data: {
					...this.locals,
				},
				context: this.context,
				...this.props,
			});
		},
	},

	unmounted() {
		this.unmounted = true;
		this.amisInstance?.unmount();
	},
};
</script>
